# Matching kernelization

***

## About

This is an implementation of maximum matching two pre-processing algorithms. The first algorithm is **degree kernelization**, which removes all vertices of degree at most two. Hence, the resulting graph will have a degree at least three. The second algorithm is **crown kernelization**, which removes all crowns. This algorithm runs in three steps.
* **Step 0**: Solve LP (find a maximum matching in an auxiliary bipartite graph)
* **Step 1**: Standard LP reduction (which removes *flared crowns*, crowns whose independent set part is strictly larger)
* **Step 2**: Further LP reduction by removing tail strongly connected components (which removes all *straight crowns* as well)

Note that step 1 and 2 run in linear time.

## How to use

### Dependencies

* Boost (v1.69.0)?

I am not sure if it has to be installed beforehand, but it works on my Macbook for now 😆

### Installation

```
git clone https://gitlab.tubit.tu-berlin.de/akt-research/crown-matching.git
cd crown-matching
mkdir build
```

### Build

```
# Within build/ directory
cmake ../
make 
```

### Run

```
./main --input <input_file>
        [--output <output_file>] [--without-kdeg] [--without-kcrown]
        [-s | --solve] [-c | --compare] [-d, | --debug]
```

#### Description

* **input_file**: An input graph in dimacs format. You must give this option.
* **--output=output_file**:  If given, the program will write the result of the kernelization to the designated path.
* **--without-kdeg**: No degree kernelization
* **--without-kcrown**: No crown kernelization
* **-s, --solve**: Solve maximum matching after kernelization using blossom5 library.
* **-c, --compare**: If both **-s** and **-c** are given, solve maximum matching without kernelization using blossom5 library (for running time comparison).
* **-d, --debug** Show debug messages.

### Output

In order to make the serialization easy, the output will be in YAML format. 

### Notes

This implementation crashes with segmentation faults for some large datasets because of a large number of recursions. In such cases, setting the stack size larger:

```
ulimit -s unlimited
```