#!/usr/bin/env python3

from argparse import ArgumentParser
from pathlib import Path
from collections import defaultdict
import subprocess
import traceback
import yaml
import csv


CWD = Path.cwd()
EXE = CWD / 'matching'
SNAP_DATA_DIR = CWD.parent / 'data'
K_SNAP_DATA_DIR = CWD.parent / 'kernel'
TIMEOUT = 3600  # in seconds


def run(command, stdin, timeout=False):
    output = subprocess.run(command, stdin=stdin, stdout=subprocess.PIPE, encoding='ascii', timeout=TIMEOUT)
    output = output.stdout
    print(output)
    return output


def run_all(dir, key_prefix=''):
    for input_path in dir.glob('*.dimacs'):
        try:
            print('Start:', input_path)
            graph_name = input_path.stem

            if not results[graph_name]:
                results[graph_name] = {}

            with open(input_path, 'r') as f:
                raw = run((EXE, ), f)
                rs = yaml.safe_load(raw)
                for k, v in rs.items():
                    results[graph_name][key_prefix + k] = v
            
            print(results)
        except Exception as e:
            print(f'Error: {str(e)}')
            print(traceback.format_exc())
            continue


if __name__ == '__main__':
    parser = ArgumentParser()
    parser.add_argument('--output', default='results.csv')
    args = parser.parse_args()

    results = defaultdict(dict)
    run_all(SNAP_DATA_DIR)
    run_all(K_SNAP_DATA_DIR, key_prefix='k_')
    print(results)

    d = []
    for k, v in results.items():
        dic = v.copy()
        dic['filename'] = k
        d.append(dic)

    keys = ['filename', 'c_time', 'c_matchings', 'k_c_time', 'k_c_matchings']
    with open(args.output, 'w') as f:
        dict_writer = csv.DictWriter(f, keys)
        dict_writer.writeheader()
        dict_writer.writerows(d)
