#include <algorithm>
#include <list>
#include "graph.hpp"

#include <iostream>

#define MAX_CORE_TO_EXAMINE 5

namespace analyzer {
  struct degeneracy_result {
    unsigned int degeneracy;
    unsigned int cores[MAX_CORE_TO_EXAMINE + 1];
  };

  struct degeneracy {

    degeneracy(graph::graph &g)
      : g(&g), degree(g.N()), deleted(g.N(), false), cursor(g.N()), bucket(g.N(), std::list<unsigned int>(0)){}

    degeneracy_result run() {
      degeneracy_result result;
      result.degeneracy = 0;

      for (auto &v: *g) {
        degree[v.id] = v.degree;
        bucket[v.degree].push_front(v.id);
        cursor[v.id] = bucket[v.degree].begin();
      }

      unsigned int k;
      for (k = 0; bucket[k].size() == 0; k++) {}

      for (size_t i = g->N(); i > 0;) {
        if (bucket[k].size() == 0) {
          k++;
          continue;
        }

        // remove a vertex with degree k
        auto vid = bucket[k].front();
        bucket[k].pop_front();
        auto v = g->get_vertex(vid);
        result.degeneracy = std::max(result.degeneracy, k);
        deleted[vid] = true;

        // delete exaclty n times
        i--;

        snapshot.push_back(k);

        for (auto &nv: v.neighbors) {
          if (deleted[nv.id]) continue;

          // move neighbor one to the left
          bucket[degree[nv.id]--].erase(cursor[nv.id]);
          bucket[degree[nv.id]].push_front(nv.id);
          cursor[nv.id] = bucket[degree[nv.id]].begin();

          // checkt nv next if degree of nv is smaller than k
          k = std::min(k, degree[nv.id]);
        }
      }

      for (size_t i = 1; i <= MAX_CORE_TO_EXAMINE; i++) {
        auto index = std::distance(snapshot.begin(), find(snapshot.begin(), snapshot.end(), i));
        result.cores[i] = g->N() - index;
      }

      return result;
    }

  private:
    graph::graph *g;
    std::vector<unsigned int> degree, snapshot;  // current degree of the graph
    std::vector<bool> deleted;
    std::vector<std::list<unsigned int>::iterator> cursor;  // for constnat time deletion
    // bucket[i] stores vertices of current degree i.
    std::vector<std::list<unsigned int>> bucket;
  };

}