/*
*
*	Author: Philipp Zschoche, https://zschoche.org
*
*/
#ifndef __LOGGER_HPP__
#define __LOGGER_HPP__

#include <chrono>
#include <iomanip>
#include <iostream>
constexpr size_t FW = 9;
#ifdef LOGGING 
#define LOG(X) { X } 
#else
#define LOG(X) 
#endif

#define LOG_RESULT(fmt, ...) { \
  printf("[Result] ");         \
  printf(fmt, ##__VA_ARGS__);  \
}

#if defined(RESULTV) || defined(RESULTVV)
#define LOG_RESULTV(fmt, ...) {  \
  printf("[Result v] ");         \
  printf(fmt, ##__VA_ARGS__);    \
}
#else
#define LOG_RESULTV(fmt, ...)
#endif

#ifdef RESULTVV
#define LOG_RESULTVV(fmt, ...) {  \
  printf("[Result vv] ");         \
  printf(fmt, ##__VA_ARGS__);     \
}
#else
#define LOG_RESULTVV(fmt, ...)
#endif

#define LOG_PERFORMANCE(fmt, ...) {  \
  printf("[Performance] ");          \
  printf(fmt, ##__VA_ARGS__);        \
}

typedef std::chrono::high_resolution_clock Clock;
typedef std::chrono::milliseconds::rep Milliseconds;
#define duration(x) std::chrono::duration_cast<std::chrono::milliseconds>(x).count()


#endif /* __LOGGER_HPP__ */

