#include <algorithm>
#include "graph.hpp"

#include <iostream>
#define MAX_DEGREE_TO_EXAMINE 5
#define MIN_IGNORED_DEGREE 500

namespace analyzer {
  struct misc_result {
    size_t n, m;
    size_t ns[MAX_DEGREE_TO_EXAMINE + 1];
    unsigned int maxDegree;
    float averageDegree;
    unsigned int octBound, hindex;
  };

  struct misc {

    misc(graph::graph &g) : g(&g), degrees(g.N()) {}

    misc_result run() {
      misc_result result;

      result.n = g->N();
      result.m = g->edgeset().M();
      for (size_t i = 0; i <= MAX_DEGREE_TO_EXAMINE; ++i) {
        result.ns[i] = 0;
      }
      result.maxDegree = 0;
      result.averageDegree = 0;
      result.octBound = 0;
      result.hindex = 0;

      for (auto &v: *g) {
        if (v.degree <= MAX_DEGREE_TO_EXAMINE) {
          result.ns[v.degree]++;
        }
        result.maxDegree = std::max(result.maxDegree, v.degree);
        result.averageDegree += v.degree;
      }
      result.averageDegree /= float(g->N());

      // Count disjoint triangles
      // We partially ignore large degree vertices to reduce running time.
      auto h = graph::exclude_isolated_vertices(*g);
      deleted.assign(h.N(), false);
      for (auto &v: h) {
        if (deleted[v.id]) continue;
        if (v.degree > MIN_IGNORED_DEGREE) continue;
        mark.assign(h.N(), false);

        for (auto &nv: v.neighbors) {
          mark[nv.id] = true;
        }

        [&] {
          for (auto &_nv: v.neighbors) {
            auto nv = h.get_vertex(_nv.id);
            if (deleted[nv.id]) continue;
            if (nv.degree > MIN_IGNORED_DEGREE) continue;
            for (auto& _nnv: nv.neighbors) {
              auto nnv = h.get_vertex(_nnv.id);
              if (not deleted[nnv.id] and mark[nnv.id]) {
                result.octBound++;
                deleted[v.id] = true;
                deleted[nv.id] = true;
                deleted[nnv.id] = true;
                return;  // break out of loop
              }
            }
          }
        }();

      }

      for (auto &v: *g) {
        degrees[v.degree]++;
      }

      unsigned int s = 0;
      for (size_t i = g->N() - 1; i >= 0; i--) {
        s += degrees[i];
        if (s >= i) {
          result.hindex = i;
          break;
        }
      }

      return result;
    }

  private:
    graph::graph *g;
    std::vector<bool> deleted, mark;
    std::vector<int> degrees;
  };
}
