#include <vector>
#include <algorithm>
#include "graph.hpp"


namespace kernelizer {

  // Find strongly connected components using Kosaruju's algorithm
  struct scc {
    scc(graph::graph &g, std::vector<int> &matchedRL, std::vector<int> &matchedLR)
      : g(&g), n(g.N()), matchedRL(matchedRL), matchedLR(matchedLR),
        visited(2 * g.N(), false), topologicalOrder(2 * g.N(), -1) {}

    // Returns the topological ranking of the SCC each vertex belongs to
    std::vector<int> build() {
      for (int lr = 0; lr < 2; ++lr) {
        for (auto &v: *g) {
          fdfs(v.id, lr);
        }
      }
      reverse(visitOrder.begin(), visitOrder.end());

      int count = 0;
      for (int vlr: visitOrder) {
        if (topologicalOrder[vlr] < 0) {
          int v = vlr % n;
          int lr = vlr / n;
          rdfs(v, lr, count++);
        }
      }

      return topologicalOrder;
    }

  private:
    graph::graph *g;
    int n;
    std::vector<int> matchedLR, matchedRL, visitOrder, topologicalOrder;
    std::vector<bool> visited;

    // forward pass
    void fdfs(int v, int lr) {
      int vlr = v + n * lr;
      if (visited[vlr]) return;
      visited[vlr] = true;
      if (lr and matchedRL[v] >= 0) {  // right side
        fdfs(matchedRL[v], 0);
      } else {
        for (auto &vv: g->get_vertex(v).neighbors) {
          fdfs(vv.id, 1);
        }
      }
      visitOrder.push_back(vlr);
    }

    // reverse pass
    void rdfs(int v, int lr, int count) {
      int vlr = v + n * lr;
      if (topologicalOrder[vlr] >= 0) return;
      topologicalOrder[vlr] = count;
      if (not lr and matchedLR[v] >= 0) {  // left side
        rdfs(matchedLR[v], 1, count);
      } else {
        for (auto &vv: g->get_vertex(v).neighbors) {
          rdfs(vv.id, 0, count);
        }
      }
    }

  };
}