#define BOOST_TEST_MODULE ext2tests
// svn test

#include <boost/test/unit_test.hpp>
#include <fstream>
#include <iostream>
#include "mmatching.hpp"
#include "graph.hpp"

BOOST_AUTO_TEST_CASE(boost_test_test) { BOOST_REQUIRE_EQUAL(true, true); }

BOOST_AUTO_TEST_CASE(graph_test) {
	std::ifstream f("../data/facebook.txt");
	auto g = akt::parse_snap(f,4039, 88234);
	int i=1;
	for(auto& v : g[0].neighbors) {
		BOOST_REQUIRE_EQUAL(i, v.id);
		i++;
	}
	BOOST_REQUIRE_EQUAL(i, 348);
	BOOST_REQUIRE_EQUAL(g[0].degree, 347);
	auto iter =g[1].neighbors.begin();
	BOOST_REQUIRE_EQUAL(iter->id, 0);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 48);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 53);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 54);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 73);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 88);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 92);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 119);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 126);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 133);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 194);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 236);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 280);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 299);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 315);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 322);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 346);
	++iter;
	BOOST_CHECK(iter == g[1].neighbors.end());

}


BOOST_AUTO_TEST_CASE(graph_test_remove_edge) {
	
	std::ifstream f("../data/facebook.txt");
	auto g = akt::parse_snap(f,4039, 88234);
	bool found = false;
	for(auto& v : g[1].neighbors) {
		if(v.id == 322) found = true;
	}
	BOOST_REQUIRE_EQUAL(found, true);
	found = false;
	for(auto& v : g[322].neighbors) {
		if(v.id == 1) found = true;
	}
	BOOST_REQUIRE_EQUAL(found, true);
	for(auto& e : g.edgeset()) {
		if(e.x.id == 1 && e.y.id == 322) {
			found = true;
			g.remove_edge(e);
		}
	}
	BOOST_REQUIRE_EQUAL(found, true);
	found = false;
	for(auto& e : g.edgeset()) {
		if(e.x.id == 1 && e.y.id == 322) {
			found = true;
		}
	}
	BOOST_REQUIRE_EQUAL(found, false);
	for(auto& v : g[1].neighbors) {
		if(v.id == 322) found = true;
	}
	BOOST_REQUIRE_EQUAL(found, false);
	for(auto& v : g[322].neighbors) {
		if(v.id == 1) found = true;
	}
	BOOST_REQUIRE_EQUAL(found, false);

	auto iter =g[1].neighbors.begin();
	BOOST_REQUIRE_EQUAL(iter->id, 0);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 48);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 53);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 54);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 73);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 88);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 92);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 119);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 126);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 133);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 194);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 236);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 280);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 299);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 315);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 346);
	++iter;
	BOOST_CHECK(iter == g[1].neighbors.end());
	for(auto& e : g.edgeset()) {
		if(e.x.id == 1 && e.y.id == 346 ) {
			g.remove_edge(e);
		}
		if(e.x.id == 0 && e.y.id == 1) {
			g.remove_edge(e);
		}
	}
	iter =g[1].neighbors.begin();
	BOOST_REQUIRE_EQUAL(iter->id, 48);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 53);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 54);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 73);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 88);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 92);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 119);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 126);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 133);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 194);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 236);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 280);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 299);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 315);
	++iter;
	BOOST_CHECK(iter == g[1].neighbors.end());
	for(auto& e : g.edgeset()) {
		if(e.x.id == 1 || e.y.id == 1 ) {
			g.remove_edge(e);
		}
	}
	BOOST_CHECK(g[1].neighbors.begin() == g[1].neighbors.end());
	BOOST_REQUIRE_EQUAL(g[1].degree, 0);


}


BOOST_AUTO_TEST_CASE(graph_test_remove_vertex) {
	
	std::ifstream f("../data/facebook.txt");
	auto g = akt::parse_snap(f,4039, 88234);
	auto g2 = exclude_vertices(g,[](auto& v) { return v.id == 0; });
	auto iter =g2[0].neighbors.begin();
	BOOST_REQUIRE_EQUAL(iter->id, 47);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 52);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 53);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 72);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 87);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 91);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 118);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 125);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 132);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 193);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 235);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 279);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 298);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 314);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 321);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 345);
	++iter;
	BOOST_CHECK(iter == g2[0].neighbors.end());
	auto g3 = exclude_vertices(g,[](auto& v) { return v.id == 3; });
	g3 = exclude_vertices(g3,[](auto& v) { return v.id == 2; });
	g3 = exclude_vertices(g3,[](auto& v) { return v.id == 1; });
	g3 = exclude_vertices(g3,[](auto& v) { return v.id == 0; });
	g3 = exclude_vertices(g3,[](auto& v) { return v.id == 4038; });
	for(int i = 0; i < g3.N() ; i++) {
		auto iter = g[i+4].neighbors.begin();
		auto iter3 = g3[i].neighbors.begin();
		while(iter !=g[i+4].neighbors.end() && iter3 != g3[i].neighbors.end()) {
			if(iter->id <= 3 || iter->id == 4038) {
				++iter;
			} else {
				BOOST_REQUIRE_EQUAL(iter->id - 4, iter3->id);
				++iter;
				++iter3;
			}
		}
	}
}


BOOST_AUTO_TEST_CASE(blossom5_test) {
	using namespace mmatching;
	{
		std::ifstream f("../data/as20000102.txt.dimacs");
		auto g = akt::parse_dimacs(f);
		BOOST_REQUIRE_EQUAL(blossom5::maximum_matching_size(g.edgeset().begin(),g.edgeset().end(),g.N(),g.edgeset().M()), 1048);
	}

	{
		std::ifstream f("../data/p2p-Gnutella04.txt.dimacs");
		auto g = akt::parse_dimacs(f);
		BOOST_REQUIRE_EQUAL(blossom5::maximum_matching_size(g.edgeset().begin(),g.edgeset().end(),g.N(),g.edgeset().M()), 3170);
	}

	{
		std::ifstream f("../data/CA-HepTh.txt.dimacs");
		auto g = akt::parse_dimacs(f);
		BOOST_REQUIRE_EQUAL(blossom5::maximum_matching_size(g.edgeset().begin(),g.edgeset().end(),g.N(),g.edgeset().M()), 4457);
	}
	{
		std::ifstream f("../data/p2p-Gnutella05.txt.dimacs");
		auto g = akt::parse_dimacs(f);
		BOOST_REQUIRE_EQUAL(blossom5::maximum_matching_size(g.edgeset().begin(),g.edgeset().end(),g.N(),g.edgeset().M()), 3428);
	}
	{
		std::ifstream f("../data/p2p-Gnutella06.txt.dimacs");
		auto g = akt::parse_dimacs(f);
		BOOST_REQUIRE_EQUAL(blossom5::maximum_matching_size(g.edgeset().begin(),g.edgeset().end(),g.N(),g.edgeset().M()), 3405);
	}
	{
		std::ifstream f("../data/p2p-Gnutella08.txt.dimacs");
		auto g = akt::parse_dimacs(f);
		BOOST_REQUIRE_EQUAL(blossom5::maximum_matching_size(g.edgeset().begin(),g.edgeset().end(),g.N(),g.edgeset().M()), 2054);
	}
	{
		std::ifstream f("../data/p2p-Gnutella09.txt.dimacs");
		auto g = akt::parse_dimacs(f);
		BOOST_REQUIRE_EQUAL(blossom5::maximum_matching_size(g.edgeset().begin(),g.edgeset().end(),g.N(),g.edgeset().M()), 2574);
	}

}	

BOOST_AUTO_TEST_CASE(rr_test1) {
	std::ifstream f("../data/as20000102.txt.dimacs");
	auto g = akt::parse_dimacs(f);
	auto s = mmatching::kernelizer(g).isolate();
	BOOST_REQUIRE_EQUAL(s, 1048);
	BOOST_REQUIRE_EQUAL(mmatching::blossom5::maximum_matching_size(g.edgeset().begin(),g.edgeset().end(),g.N(),g.edgeset().M())+s, 1048);
	BOOST_REQUIRE_EQUAL(mmatching::blossom5::rnd::maximum_matching_size(g.edgeset().begin(),g.edgeset().end(),g.N(),g.edgeset().M())+s, 1048);
	auto g2 = exclude_isolated_vertices(g);
	BOOST_REQUIRE_EQUAL(g2.N(), 0);
	BOOST_REQUIRE_EQUAL(mmatching::blossom5::maximum_matching_size(g2.edgeset().begin(),g2.edgeset().end(),g2.N(),g2.edgeset().M())+s, 1048);
	BOOST_REQUIRE_EQUAL(mmatching::blossom5::rnd::maximum_matching_size(g2.edgeset().begin(),g2.edgeset().end(),g2.N(),g2.edgeset().M())+s, 1048);
	

}
BOOST_AUTO_TEST_CASE(rr_test2) {
	std::ifstream f("../data/p2p-Gnutella04.txt.dimacs");
	auto g = akt::parse_dimacs(f);

	auto a = mmatching::blossom5::maximum_matching_size(g.edgeset().begin(),g.edgeset().end(),g.N(),g.edgeset().M());
	auto d = mmatching::blossom5::rnd::maximum_matching_size(g.edgeset().begin(),g.edgeset().end(),g.N(),g.edgeset().M());
	auto s = mmatching::kernelizer(g).isolate();
	auto b = mmatching::blossom5::maximum_matching_size(g.edgeset().begin(),g.edgeset().end(),g.N(),g.edgeset().M());
	BOOST_REQUIRE_EQUAL(a, s+b);
	BOOST_REQUIRE_EQUAL(a, d);
	auto g2 = exclude_isolated_vertices(g);
	auto c = mmatching::blossom5::maximum_matching_size(g2.edgeset().begin(),g2.edgeset().end(),g2.N(),g2.edgeset().M());
	BOOST_REQUIRE_EQUAL(c, b);
}

BOOST_AUTO_TEST_CASE(rr_test3) {
		std::ifstream f("../data/CA-HepTh.txt.dimacs");
		auto g = akt::parse_dimacs(f);

	auto a = mmatching::blossom5::maximum_matching_size(g.edgeset().begin(),g.edgeset().end(),g.N(),g.edgeset().M());
	auto d = mmatching::blossom5::rnd::maximum_matching_size(g.edgeset().begin(),g.edgeset().end(),g.N(),g.edgeset().M());
	auto s = mmatching::kernelizer(g).isolate();
	auto b = mmatching::blossom5::maximum_matching_size(g.edgeset().begin(),g.edgeset().end(),g.N(),g.edgeset().M());
	BOOST_REQUIRE_EQUAL(a, s+b);
	BOOST_REQUIRE_EQUAL(a, d);
	auto g2 = exclude_isolated_vertices(g);
	auto c = mmatching::blossom5::maximum_matching_size(g2.edgeset().begin(),g2.edgeset().end(),g2.N(),g2.edgeset().M());
	BOOST_REQUIRE_EQUAL(c, b);
}

BOOST_AUTO_TEST_CASE(rr_test4) {
		std::ifstream f("../data/p2p-Gnutella05.txt.dimacs");
		auto g = akt::parse_dimacs(f);

	auto a = mmatching::blossom5::maximum_matching_size(g.edgeset().begin(),g.edgeset().end(),g.N(),g.edgeset().M());
	auto d = mmatching::blossom5::rnd::maximum_matching_size(g.edgeset().begin(),g.edgeset().end(),g.N(),g.edgeset().M());
	auto s = mmatching::kernelizer(g).isolate();
	auto b = mmatching::blossom5::maximum_matching_size(g.edgeset().begin(),g.edgeset().end(),g.N(),g.edgeset().M());
	BOOST_REQUIRE_EQUAL(a, s+b);
	BOOST_REQUIRE_EQUAL(a, d);
	auto g2 = exclude_isolated_vertices(g);
	auto c = mmatching::blossom5::maximum_matching_size(g2.edgeset().begin(),g2.edgeset().end(),g2.N(),g2.edgeset().M());
	BOOST_REQUIRE_EQUAL(c, b);
}

BOOST_AUTO_TEST_CASE(rr_test5) {
		std::ifstream f("../data/p2p-Gnutella06.txt.dimacs");
		auto g = akt::parse_dimacs(f);

	auto a = mmatching::blossom5::maximum_matching_size(g.edgeset().begin(),g.edgeset().end(),g.N(),g.edgeset().M());
	auto d = mmatching::blossom5::rnd::maximum_matching_size(g.edgeset().begin(),g.edgeset().end(),g.N(),g.edgeset().M());
	auto s = mmatching::kernelizer(g).isolate();
	auto b = mmatching::blossom5::maximum_matching_size(g.edgeset().begin(),g.edgeset().end(),g.N(),g.edgeset().M());
	BOOST_REQUIRE_EQUAL(a, s+b);
	BOOST_REQUIRE_EQUAL(a, d);
	auto g2 = exclude_isolated_vertices(g);
	auto c = mmatching::blossom5::maximum_matching_size(g2.edgeset().begin(),g2.edgeset().end(),g2.N(),g2.edgeset().M());
	BOOST_REQUIRE_EQUAL(c, b);
}

BOOST_AUTO_TEST_CASE(rr_test6) {
		std::ifstream f("../data/p2p-Gnutella08.txt.dimacs");
		auto g = akt::parse_dimacs(f);

	auto a = mmatching::blossom5::maximum_matching_size(g.edgeset().begin(),g.edgeset().end(),g.N(),g.edgeset().M());
	auto d = mmatching::blossom5::rnd::maximum_matching_size(g.edgeset().begin(),g.edgeset().end(),g.N(),g.edgeset().M());
	auto s = mmatching::kernelizer(g).isolate();
	auto b = mmatching::blossom5::maximum_matching_size(g.edgeset().begin(),g.edgeset().end(),g.N(),g.edgeset().M());
	BOOST_REQUIRE_EQUAL(a, s+b);
	BOOST_REQUIRE_EQUAL(a, d);
	auto g2 = exclude_isolated_vertices(g);
	auto c = mmatching::blossom5::maximum_matching_size(g2.edgeset().begin(),g2.edgeset().end(),g2.N(),g2.edgeset().M());
	BOOST_REQUIRE_EQUAL(c, b);
}
BOOST_AUTO_TEST_CASE(rr_test7) {
		std::ifstream f("../data/p2p-Gnutella09.txt.dimacs");
		auto g = akt::parse_dimacs(f);

	auto a = mmatching::blossom5::maximum_matching_size(g.edgeset().begin(),g.edgeset().end(),g.N(),g.edgeset().M());
	auto d = mmatching::blossom5::rnd::maximum_matching_size(g.edgeset().begin(),g.edgeset().end(),g.N(),g.edgeset().M());
	auto s = mmatching::kernelizer(g).isolate();
	auto b = mmatching::blossom5::maximum_matching_size(g.edgeset().begin(),g.edgeset().end(),g.N(),g.edgeset().M());
	BOOST_REQUIRE_EQUAL(a, s+b);
	BOOST_REQUIRE_EQUAL(a, d);
	auto g2 = exclude_isolated_vertices(g);
	auto c = mmatching::blossom5::maximum_matching_size(g2.edgeset().begin(),g2.edgeset().end(),g2.N(),g2.edgeset().M());
	BOOST_REQUIRE_EQUAL(c, b);
}
BOOST_AUTO_TEST_CASE(rr_test8) {
		std::ifstream f("../data/p2p-Gnutella25.txt.dimacs");
		auto g = akt::parse_dimacs(f);

	auto a = mmatching::blossom5::maximum_matching_size(g.edgeset().begin(),g.edgeset().end(),g.N(),g.edgeset().M());
	auto d = mmatching::blossom5::rnd::maximum_matching_size(g.edgeset().begin(),g.edgeset().end(),g.N(),g.edgeset().M());
	auto s = mmatching::kernelizer(g).isolate();
	auto b = mmatching::blossom5::maximum_matching_size(g.edgeset().begin(),g.edgeset().end(),g.N(),g.edgeset().M());
	BOOST_REQUIRE_EQUAL(a, s+b);
	BOOST_REQUIRE_EQUAL(a, d);
	auto g2 = exclude_isolated_vertices(g);
	auto c = mmatching::blossom5::maximum_matching_size(g2.edgeset().begin(),g2.edgeset().end(),g2.N(),g2.edgeset().M());
	BOOST_REQUIRE_EQUAL(c, b);
}

BOOST_AUTO_TEST_CASE(rnd_test) {
	std::ifstream f("../data/p2p-Gnutella05.txt.dimacs");
	std::ifstream fr("../data/p2p-Gnutella05.txt.dimacs");
	auto g = akt::parse_dimacs(f);
	auto gr = akt::parse_dimacs(fr);

	auto a = mmatching::blossom5::maximum_matching_size(g.edgeset().begin(),g.edgeset().end(),g.N(),g.edgeset().M());
	auto ar = mmatching::blossom5::maximum_matching_size(gr.edgeset().begin(),gr.edgeset().end(),gr.N(),gr.edgeset().M());
	BOOST_REQUIRE_EQUAL(a, ar);

	auto s = mmatching::kernelizer(g).isolate();
	auto sr = mmatching::kernelizer(gr).isolate();
	BOOST_REQUIRE_EQUAL(s, sr);

	auto b = mmatching::blossom5::maximum_matching_size(g.edgeset().begin(),g.edgeset().end(),g.N(),g.edgeset().M());
	auto br = mmatching::blossom5::maximum_matching_size(gr.edgeset().begin(),gr.edgeset().end(),gr.N(),gr.edgeset().M());
	BOOST_REQUIRE_EQUAL(b, br);

	BOOST_REQUIRE_EQUAL(a, s+b);
	auto g2 = exclude_isolated_vertices(g);
	auto g2r = exclude_isolated_vertices(gr);
	auto c = mmatching::blossom5::maximum_matching_size(g2.edgeset().begin(),g2.edgeset().end(),g2.N(),g2.edgeset().M());
	auto cr = mmatching::blossom5::maximum_matching_size(g2r.edgeset().begin(),g2r.edgeset().end(),g2r.N(),g2r.edgeset().M());
	BOOST_REQUIRE_EQUAL(c, cr);

	BOOST_REQUIRE_EQUAL(c, b);
}
BOOST_AUTO_TEST_CASE(rnd_test2) {
	std::ifstream f("../data/p2p-Gnutella06.txt.dimacs");
	std::ifstream fr("../data/p2p-Gnutella06.txt.dimacs");
	auto g = akt::parse_dimacs(f);
	auto gr = akt::parse_dimacs(fr);

	auto a = mmatching::blossom5::maximum_matching_size(g.edgeset().begin(),g.edgeset().end(),g.N(),g.edgeset().M());
	auto ar = mmatching::blossom5::maximum_matching_size(gr.edgeset().begin(),gr.edgeset().end(),gr.N(),gr.edgeset().M());
	BOOST_REQUIRE_EQUAL(a, ar);

	auto s = mmatching::kernelizer(g).isolate();
	auto sr = mmatching::kernelizer(gr).isolate();
	BOOST_REQUIRE_EQUAL(s, sr);

	auto b = mmatching::blossom5::maximum_matching_size(g.edgeset().begin(),g.edgeset().end(),g.N(),g.edgeset().M());
	auto br = mmatching::blossom5::maximum_matching_size(gr.edgeset().begin(),gr.edgeset().end(),gr.N(),gr.edgeset().M());
	BOOST_REQUIRE_EQUAL(b, br);

	BOOST_REQUIRE_EQUAL(a, s+b);
	auto g2 = exclude_isolated_vertices(g);
	auto g2r = exclude_isolated_vertices(gr);
	auto c = mmatching::blossom5::maximum_matching_size(g2.edgeset().begin(),g2.edgeset().end(),g2.N(),g2.edgeset().M());
	auto cr = mmatching::blossom5::maximum_matching_size(g2r.edgeset().begin(),g2r.edgeset().end(),g2r.N(),g2r.edgeset().M());
	BOOST_REQUIRE_EQUAL(c, cr);

	BOOST_REQUIRE_EQUAL(c, b);
}
BOOST_AUTO_TEST_CASE(rnd_test3) {
	std::ifstream f("../data/p2p-Gnutella08.txt.dimacs");
	std::ifstream fr("../data/p2p-Gnutella08.txt.dimacs");
	auto g = akt::parse_dimacs(f);
	auto gr = akt::parse_dimacs(fr);

	auto a = mmatching::blossom5::maximum_matching_size(g.edgeset().begin(),g.edgeset().end(),g.N(),g.edgeset().M());
	auto ar = mmatching::blossom5::maximum_matching_size(gr.edgeset().begin(),gr.edgeset().end(),gr.N(),gr.edgeset().M());
	BOOST_REQUIRE_EQUAL(a, ar);

	auto s = mmatching::kernelizer(g).isolate();
	auto sr = mmatching::kernelizer(gr).isolate();
	BOOST_REQUIRE_EQUAL(s, sr);

	auto b = mmatching::blossom5::maximum_matching_size(g.edgeset().begin(),g.edgeset().end(),g.N(),g.edgeset().M());
	auto br = mmatching::blossom5::maximum_matching_size(gr.edgeset().begin(),gr.edgeset().end(),gr.N(),gr.edgeset().M());
	BOOST_REQUIRE_EQUAL(b, br);

	BOOST_REQUIRE_EQUAL(a, s+b);
	auto g2 = exclude_isolated_vertices(g);
	auto g2r = exclude_isolated_vertices(gr);
	auto c = mmatching::blossom5::maximum_matching_size(g2.edgeset().begin(),g2.edgeset().end(),g2.N(),g2.edgeset().M());
	auto cr = mmatching::blossom5::maximum_matching_size(g2r.edgeset().begin(),g2r.edgeset().end(),g2r.N(),g2r.edgeset().M());
	BOOST_REQUIRE_EQUAL(c, cr);

	BOOST_REQUIRE_EQUAL(c, b);
}
BOOST_AUTO_TEST_CASE(rnd_test4) {
	std::ifstream f("../data/p2p-Gnutella09.txt.dimacs");
	std::ifstream fr("../data/p2p-Gnutella09.txt.dimacs");
	auto g = akt::parse_dimacs(f);
	auto gr = akt::parse_dimacs(fr);

	auto a = mmatching::blossom5::maximum_matching_size(g.edgeset().begin(),g.edgeset().end(),g.N(),g.edgeset().M());
	auto ar = mmatching::blossom5::maximum_matching_size(gr.edgeset().begin(),gr.edgeset().end(),gr.N(),gr.edgeset().M());
	BOOST_REQUIRE_EQUAL(a, ar);

	auto s = mmatching::kernelizer(g).isolate();
	auto sr = mmatching::kernelizer(gr).isolate();
	BOOST_REQUIRE_EQUAL(s, sr);

	auto b = mmatching::blossom5::maximum_matching_size(g.edgeset().begin(),g.edgeset().end(),g.N(),g.edgeset().M());
	auto br = mmatching::blossom5::maximum_matching_size(gr.edgeset().begin(),gr.edgeset().end(),gr.N(),gr.edgeset().M());
	BOOST_REQUIRE_EQUAL(b, br);

	BOOST_REQUIRE_EQUAL(a, s+b);
	auto g2 = exclude_isolated_vertices(g);
	auto g2r = exclude_isolated_vertices(gr);
	auto c = mmatching::blossom5::maximum_matching_size(g2.edgeset().begin(),g2.edgeset().end(),g2.N(),g2.edgeset().M());
	auto cr = mmatching::blossom5::maximum_matching_size(g2r.edgeset().begin(),g2r.edgeset().end(),g2r.N(),g2r.edgeset().M());
	BOOST_REQUIRE_EQUAL(c, cr);

	BOOST_REQUIRE_EQUAL(c, b);
}
BOOST_AUTO_TEST_CASE(rnd_test5) {
	std::ifstream f("../data/p2p-Gnutella25.txt.dimacs");
	std::ifstream fr("../data/p2p-Gnutella25.txt.dimacs");
	auto g = akt::parse_dimacs(f);
	auto gr = akt::parse_dimacs(fr);

	auto a = mmatching::blossom5::maximum_matching_size(g.edgeset().begin(),g.edgeset().end(),g.N(),g.edgeset().M());
	auto ar = mmatching::blossom5::maximum_matching_size(gr.edgeset().begin(),gr.edgeset().end(),gr.N(),gr.edgeset().M());
	BOOST_REQUIRE_EQUAL(a, ar);

	auto s = mmatching::kernelizer(g).isolate();
	auto sr = mmatching::kernelizer(gr).isolate();
	BOOST_REQUIRE_EQUAL(s, sr);

	auto b = mmatching::blossom5::maximum_matching_size(g.edgeset().begin(),g.edgeset().end(),g.N(),g.edgeset().M());
	auto br = mmatching::blossom5::maximum_matching_size(gr.edgeset().begin(),gr.edgeset().end(),gr.N(),gr.edgeset().M());
	BOOST_REQUIRE_EQUAL(b, br);

	BOOST_REQUIRE_EQUAL(a, s+b);
	auto g2 = exclude_isolated_vertices(g);
	auto g2r = exclude_isolated_vertices(gr);
	auto c = mmatching::blossom5::maximum_matching_size(g2.edgeset().begin(),g2.edgeset().end(),g2.N(),g2.edgeset().M());
	auto cr = mmatching::blossom5::maximum_matching_size(g2r.edgeset().begin(),g2r.edgeset().end(),g2r.N(),g2r.edgeset().M());
	BOOST_REQUIRE_EQUAL(c, cr);

	BOOST_REQUIRE_EQUAL(c, b);
}
