#!/usr/bin/env python3
from sys import stdin, stdout
from os.path import basename
import re
from collections import namedtuple
import csv


pattern = re.compile(
    '\[Result\] Load data from (?P<filepath>[A-Za-z\d\s\/\.:,-]+)'
    '\[Result\] Degree kernelization: start with # of vertices: (?P<n>\d+), # of edges: (?P<m>\d+)'
    '\[Result\] Degree kernelization: (?P<deg_matchings>\d+) matchings found'
    '\[Performance\] Degree kernelization: finished in (?P<deg_time>\d+) \(ms\)'
    '\[Result\] Crown kernelization: start with # of vertices: (?P<n_dash>\d+), # of edges: (?P<m_dash>\d+)'
    '\[Performance\] LP finished in (?P<lp_time>\d+) \(ms\)'
    '\[Performance\] Part 1 finished in (?P<part1_time>\d+) \(ms\)'
    '\[Result\] (?P<part1_matchings>\d+) matchings found in Part 1'
    '\[Result\] # of vertices: (?P<n_part1>\d+), # of edges: (?P<m_part1>\d+) after Part 1'
    '\[Performance\] Part 2 finished in (?P<part2_time>\d+) \(ms\)'
    '\[Result\] (?P<part2_matchings>\d+) matchings found in Part 2'
    '\[Result\] # of vertices: (?P<n_part2>\d+), # of edges: (?P<m_part2>\d+) after Part 2'
    '\[Result\] # of odd cycles in 2-matching: (?P<n_odd_cyles>\d+)'
    '\[Performance\] Crown kernelization: finished in (?P<total_time>\d+) \(ms\)')


if __name__ == '__main__':
    lines = ''.join(line.rstrip() for line in stdin)

    results = [m.groupdict() for m in pattern.finditer(lines)]
    for row in results:
        row['filename'] = basename(row['filepath'])
    results = sorted(results, key=lambda row: int(row['n']))

    fieldnames = ['filename', 'n', 'm', 'n_dash', 'm_dash', 'n_part2', 'm_part2', 'n_odd_cyles', 'total_time', 'n_mm', 'lp_sol']
    writer = csv.DictWriter(stdout, fieldnames=fieldnames)

    writer.writeheader()
    for result in results:
        writer.writerow({key: result[key] for key in fieldnames})
