import numpy as np
import networkx as nx
from argparse import ArgumentParser
from pathlib import Path


if __name__ == '__main__':
    parser = ArgumentParser()
    parser.add_argument('--input', type=str)
    parser.add_argument('--output', type=str)
    parser.add_argument('--times', type=int)
    args = parser.parse_args()
    file = Path(args.input)
    outdir = Path(args.output)

    edges = []
    with open(file) as f:
        for line in f:
            identifier, u, v = line.split()
            if identifier == 'p':
                n, m = u, v
            if identifier == 'e':
                edges.append((int(u), int(v)))
    G = nx.Graph()
    G.add_edges_from(edges)

    for i in range(args.times):
        with open(outdir / f'data_{i:02}.txt.dimacs', 'w') as f:
            mapping = dict(zip(G.nodes(), np.random.permutation(G.nodes())))
            H = nx.relabel_nodes(G, mapping)
            print(f'p {n} {m}', file=f)
            for u, v in np.random.permutation(H.edges()):
                print(f'e {u} {v}', file=f)
