#!/usr/bin/env python3
from pathlib import Path
import fileinput
import re

"""
Some DIMACS datasets seem to have incorrect data format.
This script will fix the errors and remove comments on the datasets.
"""

if __name__ == '__main__':
    p = Path('../dimacs')
    for f in p.iterdir():
        print(f.suffix)
    files = [f for f in p.iterdir() if f.suffix == '.clq']
    print(files)
    for f in files:
        print('Reformatting:', f.name)
        with open(f, 'r') as fp:
            lines = []
            for line in fp:
                if line.startswith('c'):
                    continue
                elif line.startswith('p'):
                    pattern = re.compile('p\s+(edge|col)\s+(?P<n>\d+)\s+(?P<m>\d+)')
                    m = re.match(pattern, line)
                    print(m)
                    lines.append('p {} {}\n'.format(m.group('n'), m.group('m')))
                else:
                    lines.append(line)
        lines = ''.join(lines)

        with open(f, 'w') as fp:
            fp.write(lines)
