#!/usr/bin/env python3

from argparse import ArgumentParser
from pathlib import Path
import subprocess
import traceback
import yaml
import csv


CWD = Path.cwd()
EXE = BUILD_DIR / 'matching'
SNAP_DATA_DIR = CWD.parent / 'data' / 'snap'
K_SNAP_DATA_DIR = CWD.parent / 'kernel' / 'snap'


def run(command, timeout=False):
    if not timeout:
        output = subprocess.run(command, stdout=subprocess.PIPE, encoding='ascii')
    else:
        output = subprocess.run(command, stdout=subprocess.PIPE, encoding='ascii', timeout=args.timeout)
    output = output.stdout
    print(output)
    return output


def run_all(dir, key_prefix=''):
    results = {}
    for input_path in SNAP_DATA_DIR.glob('*.dimacs'):
        try:
            print('Start:', input_path)
            graph_name = input_path.stem
            results[graph_name] = {}

            with open(input_path, 'r') as f:
                raw = run(EXE, stdin=f)
                rs = yaml.safe_load(raw)
                for k, v in rs.items:
                    results[key_prefix + k] = v

        except Exception as e:
            print(f'Error: {str(e)}')
            print(traceback.format_exc())
            continue

    return results


if __name__ == '__main__':

    print(run_all(SNAP_DATA_DIR))
