#!/usr/bin/env python

from argparse import ArgumentParser
from collections import defaultdict
import csv
import glob
import os, os.path
import re
import signal

import networkx as nx
from mv_algorithm_fast_test import mv_max_cardinality


CWD = os.getcwd()
CWD_PARENT = os.path.dirname(CWD)
SNAP_DATA_DIR = os.path.join(CWD_PARENT, 'data')
K_SNAP_DATA_DIR = os.path.join(CWD_PARENT, 'kernel')
DEFAULT_TIMEOUT = 3


def handler(signum, frame):
    raise Exception("Timeout!")


def run(filepath):
    edges = []
    with open(filepath, 'r') as f:
        for line in f:
            identifier, u, v = line.split()
            if identifier is 'e':
                edges.append((int(u), int(v)))
    G = nx.Graph()
    G.add_edges_from(edges)
    signal.alarm(args.timeout)
    try:
        ms, _, time = mv_max_cardinality(G, 1, False, 100)
    except Exception, exc:
        print exc
        return
    signal.alarm(0)
    return ms, time


if __name__ == '__main__':
    parser = ArgumentParser()
    parser.add_argument('--timeout', default=DEFAULT_TIMEOUT, type=int)
    parser.add_argument('--output', default='results.csv')
    args = parser.parse_args()

    results = defaultdict(dict)
    signal.signal(signal.SIGALRM, handler)
    for filepath in glob.glob(os.path.join(SNAP_DATA_DIR, '*.dimacs')):
        filename = os.path.basename(filepath)
        r = run(filepath)
        if r is not None:
            results[filename]['mv_matchings'] = r[0]
            results[filename]['mv_time'] = r[1]
        else:
            results[filename] = {}
    for filepath in glob.glob(os.path.join(K_SNAP_DATA_DIR, '*.dimacs')):
        filename = os.path.basename(filepath)
        r = run(filepath)
        if r is not None:
            results[filename]['k_mv_matchings'] = r[0]
            results[filename]['k_mv_time'] = r[1]
    data = []
    for k, v in results.items():
        vv = v.copy()
        vv['filename'] = k
        data.append(vv)
    keys = ['filename', 'mv_matchings', 'mv_time', 'k_mv_matchings', 'k_mv_time']
    with open('out.csv', 'w') as outf:
        dict_writer = csv.DictWriter(outf, keys)
        dict_writer.writeheader()
        dict_writer.writerows(data)
