#include "mmatching.hpp"
#include <boost/program_options.hpp>
#include <iostream>
#include <fstream>
#include <chrono>
#include <iomanip>
#include <cstdio>
#include "graph.hpp"
#include "weighted_kernel.hpp"

using namespace boost;
namespace po = boost::program_options;

void benchmark(akt::graph g, const std::string& filename, bool skip) {
	std::cout << std::setw(FW) << g.N() << "\t" << std::setw(FW) << g.edgeset().M() << "\t";
	std::cerr << "info: compute maximum matching..." << std::endl;
	auto begin = std::chrono::high_resolution_clock::now();
	size_t sol = 0;
	if(!skip) {
		sol = mmatching::blossom5::maximum_matching_size(g.edgeset().begin(),g.edgeset().end(),g.N(),g.edgeset().M());
	} else {
		std::cout << std::setw(FW) << 0 << '\t';
	}
	auto tsol = std::chrono::high_resolution_clock::now();
	std::cout << std::setw(FW) << std::chrono::duration_cast<std::chrono::milliseconds>(tsol-begin).count() << '\t' << std::setw(FW) <<sol << '\t';
	std::cerr << "info: maximum matching size = " << sol << std::endl;
	std::cerr << "info: kernelize..." << std::endl;
	begin = std::chrono::high_resolution_clock::now();
	mmatching::weighted_kernelizer kernel(&g);
	auto s = kernel();
	auto g2 = exclude_isolated_vertices(g);
	auto ts = std::chrono::high_resolution_clock::now();
	std::cout << std::setw(FW) <<std::chrono::duration_cast<std::chrono::milliseconds>(ts-begin).count() << '\t' << std::setw(FW) << g2.N() << '\t' << std::setw(FW) <<g.edgeset().M() << '\t' << std::setw(FW) <<s << '\t';
	std::cerr << "info: pre-matched " << s << " edges." << std::endl;
	std::cerr << "info: compute maximum matching on kernel..." << std::endl;
	begin = std::chrono::high_resolution_clock::now();
	auto sol2 =  mmatching::blossom5::maximum_matching_size(g2.edgeset().begin(),g2.edgeset().end(),g2.N(),g2.edgeset().M());
	auto tsol2 = std::chrono::high_resolution_clock::now();
	std::cout << std::setw(FW) <<std::chrono::duration_cast<std::chrono::milliseconds>(tsol2-begin).count() << '\t' << std::setw(FW) << sol2;
	if(!skip && sol != sol2 + s) { 
		std::cout << "error: got a wrong result." << std::endl;
		exit(1);
	}
	std::cerr << "info: maximum matching size on kernel = " << sol2 << std::endl;
	std::remove((filename + ".kernel").c_str());
	std::ofstream file(filename + ".kernel");
	akt::to_dimacs(file, g2);
}

int main(int ac, char *av[]) {

        po::options_description desc("usage: parser [options]\nAllowed options");
        po::positional_options_description p;
        std::string input,output;
        std::string file;
        size_t vertices,edges;
	bool skip;
        desc.add_options() 
		("help", "produce help message")
	("input-format,i", po::value<std::string>(&input)->default_value("dimacs"))
        //("output,o", po::value<std::string>(&output)->default_value("ortmann"))
        ("file,f", po::value<std::string>(&file)->default_value(""))
        ("num-vertices,n", po::value<size_t>(&vertices)->default_value(0))
        ("skip", po::value<bool>(&skip)->default_value(false))
        ("num-edges,m", po::value<size_t>(&edges)->default_value(0));

        po::variables_map vm;
        po::store(po::command_line_parser(ac, av).options(desc).positional(p).run(), vm);
        po::notify(vm);

	if (vm.count("help")) {
                std::cerr << desc << std::endl;
                std::cerr <<  std::endl;
		std::cout << "# output format:" << std::endl;
		std::cout << "# parsing,	 vertices,	    edges,   mm (parsing), 	mm (time), 	mm (size),  kernel (time), kernel (vertices), kernel (edges), kernel pre-fixed, k. mm (parsing), k. mm (time), k. mm (size), graph" << std::endl;
                return 0;
	} else if(input == "dimacs") {
                if(file.empty()) {
			std::cerr << "info: parsing..." << std::endl;
			benchmark(akt::rnd::parse_dimacs_with_weights(std::cin),"cin",skip);
		} else {

			std::cerr << "info: parsing " << file << "..." << std::endl;
                        std::ifstream f(file);
                        benchmark(akt::rnd::parse_dimacs_with_weights(f),file, skip);
                }
        }
	std::cout << '\t' << file << std::endl;
	std::cerr <<  std::endl;

        return 0;
}

