/*
*
*	Author: Philipp Zschoche, https://zschoche.org
*
*/
#ifndef __KERNEL_HPP__
#define __KERNEL_HPP__

#include "graph.hpp"
#include <queue>
namespace mmatching {

struct kernelizer {

	kernelizer(akt::graph &g) : g(&g),mark(g.N(),false) {}

	size_t isolate() {
		size_t result = 0;
		collect_vertices();

		while (!V.empty()) {
			auto *v = V.front();
			V.pop();
			if (v->degree == 1) {
				result++;
				remove_leaf(v);
			} else if (v->degree == 2) {
				result++;
				remove_redex(v);
			}
		}
		return result;
	}

      private:
	std::queue<akt::vertex *> V;

	void remove_redex(akt::vertex *v) {
		auto& u = *v->neighbors.begin();
		auto& w = *u.next;
		auto& uu =g->get_vertex(u.id);
		auto& ww =g->get_vertex(w.id);
		g->remove_edge(*u.e);
		g->remove_edge(*w.e);
		if(uu.degree < ww.degree) 
			merge(uu, ww);	
		else 
			merge(ww, uu);	
	}

	void merge(akt::vertex& u, akt::vertex& w) {
		
		for(auto& v : w.neighbors) {
			mark[v.id] = true;
		}
		auto iter = u.neighbors.begin();
		while(iter != u.neighbors.end()) {
			auto& v = (*iter);
			++iter;
			if(v.id == w.id) {
				g->remove_edge(*v.e);
			} else {
			if(mark[v.id]) {
				//just remove this edge
				g->remove_edge(*v.e);
				//check degree
				auto& vv = g->get_vertex(v.id);
				if(vv.degree <= 2) V.push(&vv);
			} else {
				//redirect edge						
				auto* c = v.complement();
				c->id = w.id;
				u.neighbors.remove(v);
				u.degree--;
				v.next = nullptr;
				w.neighbors.add(&v);
				w.degree++;
			}
			}
		}

			if(w.degree <= 2) {
				V.push(&w);
			}

		//not optimal
		iter = w.neighbors.begin();
		while(iter != w.neighbors.end()) {
			mark[iter->id] = false;
			++iter;
		}


	}

	void remove_leaf(akt::vertex *v) {
		auto &n = (*g)[v->neighbors.begin()->id];
		// isolate the deg 1 vertex
		g->remove_edge(*v->neighbors.begin()->e);

		// isolate the neighbor
		for (auto &w : n.neighbors) {
			g->remove_edge(*w.e);
			if ((*g)[w.id].degree <= 2) {
				V.push(&(*g)[w.id]); // there is a new deg 2 vertex.
			}
		}
	}

	void collect_vertices() {
		for (auto &v : *g) {
			if (v.degree == 1 || v.degree == 2)
				V.push(&v);
		}
	}
	akt::graph *g;
	std::vector<bool> mark;
};

} /* namespace mmatching */

#endif /* __KERNEL_HPP__ */
