#define BOOST_TEST_MODULE ext2tests
// svn test

#include "graph.hpp"
#include "mmatching.hpp"
#include "weighted_kernel.hpp"
#include <boost/test/unit_test.hpp>
#include <fstream>
#include <iostream>

BOOST_AUTO_TEST_CASE(boost_test_test) { BOOST_REQUIRE_EQUAL(true, true); }

BOOST_AUTO_TEST_CASE(boost_test_test2) { 

	akt::graph g(3,1);
	g.add_edge(0,1);
	g.add_edge(0,2);
}

BOOST_AUTO_TEST_CASE(graph_test) {
	std::ifstream f("../data/facebook.txt");
	auto g = akt::parse_snap(f, 4039, 88234);
	int i = 1;
	for (auto &v : g[0].neighbors) {
		BOOST_REQUIRE_EQUAL(i, v.id);
		i++;
	}
	BOOST_REQUIRE_EQUAL(i, 348);
	BOOST_REQUIRE_EQUAL(g[0].degree, 347);
	auto iter = g[1].neighbors.begin();
	BOOST_REQUIRE_EQUAL(iter->id, 0);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 48);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 53);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 54);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 73);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 88);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 92);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 119);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 126);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 133);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 194);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 236);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 280);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 299);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 315);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 322);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 346);
	++iter;
	BOOST_CHECK(iter == g[1].neighbors.end());
}

BOOST_AUTO_TEST_CASE(graph_test_remove_edge) {

	std::ifstream f("../data/facebook.txt");
	auto g = akt::parse_snap(f, 4039, 88234);
	bool found = false;
	for (auto &v : g[1].neighbors) {
		if (v.id == 322)
			found = true;
	}
	BOOST_REQUIRE_EQUAL(found, true);
	found = false;
	for (auto &v : g[322].neighbors) {
		if (v.id == 1)
			found = true;
	}
	BOOST_REQUIRE_EQUAL(found, true);
	for (auto &e : g.edgeset()) {
		if (e.x.id == 1 && e.y.id == 322) {
			found = true;
			g.remove_edge(e);
		}
	}
	BOOST_REQUIRE_EQUAL(found, true);
	found = false;
	for (auto &e : g.edgeset()) {
		if (e.x.id == 1 && e.y.id == 322) {
			found = true;
		}
	}
	BOOST_REQUIRE_EQUAL(found, false);
	for (auto &v : g[1].neighbors) {
		if (v.id == 322)
			found = true;
	}
	BOOST_REQUIRE_EQUAL(found, false);
	for (auto &v : g[322].neighbors) {
		if (v.id == 1)
			found = true;
	}
	BOOST_REQUIRE_EQUAL(found, false);

	auto iter = g[1].neighbors.begin();
	BOOST_REQUIRE_EQUAL(iter->id, 0);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 48);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 53);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 54);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 73);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 88);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 92);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 119);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 126);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 133);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 194);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 236);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 280);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 299);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 315);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 346);
	++iter;
	BOOST_CHECK(iter == g[1].neighbors.end());
	for (auto &e : g.edgeset()) {
		if (e.x.id == 1 && e.y.id == 346) {
			g.remove_edge(e);
		}
		if (e.x.id == 0 && e.y.id == 1) {
			g.remove_edge(e);
		}
	}
	iter = g[1].neighbors.begin();
	BOOST_REQUIRE_EQUAL(iter->id, 48);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 53);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 54);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 73);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 88);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 92);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 119);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 126);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 133);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 194);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 236);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 280);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 299);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 315);
	++iter;
	BOOST_CHECK(iter == g[1].neighbors.end());
	for (auto &e : g.edgeset()) {
		if (e.x.id == 1 || e.y.id == 1) {
			g.remove_edge(e);
		}
	}
	BOOST_CHECK(g[1].neighbors.begin() == g[1].neighbors.end());
	BOOST_REQUIRE_EQUAL(g[1].degree, 0);
}

BOOST_AUTO_TEST_CASE(graph_test_remove_vertex) {

	std::ifstream f("../data/facebook.txt");
	auto g = akt::parse_snap(f, 4039, 88234);
	auto g2 = exclude_vertices(g, [](auto &v) { return v.id == 0; });
	auto iter = g2[0].neighbors.begin();
	BOOST_REQUIRE_EQUAL(iter->id, 47);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 52);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 53);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 72);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 87);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 91);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 118);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 125);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 132);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 193);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 235);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 279);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 298);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 314);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 321);
	++iter;
	BOOST_REQUIRE_EQUAL(iter->id, 345);
	++iter;
	BOOST_CHECK(iter == g2[0].neighbors.end());
	auto g3 = exclude_vertices(g, [](auto &v) { return v.id == 3; });
	g3 = exclude_vertices(g3, [](auto &v) { return v.id == 2; });
	g3 = exclude_vertices(g3, [](auto &v) { return v.id == 1; });
	g3 = exclude_vertices(g3, [](auto &v) { return v.id == 0; });
	g3 = exclude_vertices(g3, [](auto &v) { return v.id == 4038; });
	for (int i = 0; i < g3.N(); i++) {
		auto iter = g[i + 4].neighbors.begin();
		auto iter3 = g3[i].neighbors.begin();
		while (iter != g[i + 4].neighbors.end() && iter3 != g3[i].neighbors.end()) {
			if (iter->id <= 3 || iter->id == 4038) {
				++iter;
			} else {
				BOOST_REQUIRE_EQUAL(iter->id - 4, iter3->id);
				++iter;
				++iter3;
			}
		}
	}
}

BOOST_AUTO_TEST_CASE(blossom5_test) {
	using namespace mmatching;
	{
		std::ifstream f("data/as20000102.txt.dimacs");
		auto g = akt::parse_dimacs(f);
		BOOST_REQUIRE_EQUAL(blossom5::maximum_matching_size(g.edgeset().begin(), g.edgeset().end(), g.N(), g.edgeset().M()), 1048);
	}

	{
		std::ifstream f("data/CA-HepTh.txt.dimacs");
		auto g = akt::parse_dimacs(f);
		BOOST_REQUIRE_EQUAL(blossom5::maximum_matching_size(g.edgeset().begin(), g.edgeset().end(), g.N(), g.edgeset().M()), 4457);
	}
	{
		std::ifstream f("data/p2p-Gnutella05.txt.dimacs");
		auto g = akt::parse_dimacs(f);
		BOOST_REQUIRE_EQUAL(blossom5::maximum_matching_size(g.edgeset().begin(), g.edgeset().end(), g.N(), g.edgeset().M()), 3428);
	}
}
BOOST_AUTO_TEST_CASE(rr_test) {
	std::mt19937 rng;
	rng.seed(std::random_device()());
	std::uniform_int_distribution<std::mt19937::result_type> dist(1, 100); //

	for (int i = 0; i < 100; i++) {
		for (int j = 0; i < 100; i++) {
			auto g = akt::generate_graph(i, dist(rng));
			auto a = mmatching::blossom5::maximum_matching_size(g.edgeset().begin(), g.edgeset().end(), g.N(), g.edgeset().M());
			auto s = mmatching::kernelizer(g).isolate();
			auto g2 = exclude_isolated_vertices(g);
			auto b = mmatching::blossom5::maximum_matching_size(g2.edgeset().begin(), g2.edgeset().end(), g2.N(), g2.edgeset().M());
			BOOST_REQUIRE_EQUAL(b + s, a);

			for (auto &v : g2) {
				BOOST_CHECK(v.degree > 2);
			}
		}
	}
}

BOOST_AUTO_TEST_CASE(rr_test1) {
	std::ifstream f("../data/as20000102.txt.dimacs");
	auto g = akt::parse_dimacs(f);
	auto s = mmatching::kernelizer(g).isolate();
	BOOST_REQUIRE_EQUAL(s, 1048);
	BOOST_REQUIRE_EQUAL(mmatching::blossom5::maximum_matching_size(g.edgeset().begin(), g.edgeset().end(), g.N(), g.edgeset().M()) + s, 1048);
	BOOST_REQUIRE_EQUAL(mmatching::blossom5::rnd::maximum_matching_size(g.edgeset().begin(), g.edgeset().end(), g.N(), g.edgeset().M()) + s, 1048);
	auto g2 = exclude_isolated_vertices(g);
	BOOST_REQUIRE_EQUAL(g2.N(), 0);
	BOOST_REQUIRE_EQUAL(mmatching::blossom5::maximum_matching_size(g2.edgeset().begin(), g2.edgeset().end(), g2.N(), g2.edgeset().M()) + s, 1048);
	BOOST_REQUIRE_EQUAL(mmatching::blossom5::rnd::maximum_matching_size(g2.edgeset().begin(), g2.edgeset().end(), g2.N(), g2.edgeset().M()) + s, 1048);
	for (auto &v : g2) {
		BOOST_CHECK(v.degree > 2);
	}
}
BOOST_AUTO_TEST_CASE(rr_test2) {
	std::ifstream f("../data/p2p-Gnutella04.txt.dimacs");
	auto g = akt::parse_dimacs(f);

	auto a = mmatching::blossom5::maximum_matching_size(g.edgeset().begin(), g.edgeset().end(), g.N(), g.edgeset().M());
	auto d = mmatching::blossom5::rnd::maximum_matching_size(g.edgeset().begin(), g.edgeset().end(), g.N(), g.edgeset().M());
	auto s = mmatching::kernelizer(g).isolate();
	auto b = mmatching::blossom5::maximum_matching_size(g.edgeset().begin(), g.edgeset().end(), g.N(), g.edgeset().M());
	BOOST_REQUIRE_EQUAL(a, s + b);
	BOOST_REQUIRE_EQUAL(a, d);
	auto g2 = exclude_isolated_vertices(g);
	auto c = mmatching::blossom5::maximum_matching_size(g2.edgeset().begin(), g2.edgeset().end(), g2.N(), g2.edgeset().M());
	BOOST_REQUIRE_EQUAL(c, b);
	for (auto &v : g2) {
		BOOST_CHECK(v.degree > 2);
	}
}

BOOST_AUTO_TEST_CASE(rr_test3) {
	std::ifstream f("../data/CA-HepTh.txt.dimacs");
	auto g = akt::parse_dimacs(f);

	auto a = mmatching::blossom5::maximum_matching_size(g.edgeset().begin(), g.edgeset().end(), g.N(), g.edgeset().M());
	auto d = mmatching::blossom5::rnd::maximum_matching_size(g.edgeset().begin(), g.edgeset().end(), g.N(), g.edgeset().M());
	auto s = mmatching::kernelizer(g).isolate();
	auto b = mmatching::blossom5::maximum_matching_size(g.edgeset().begin(), g.edgeset().end(), g.N(), g.edgeset().M());
	BOOST_REQUIRE_EQUAL(a, s + b);
	BOOST_REQUIRE_EQUAL(a, d);
	auto g2 = exclude_isolated_vertices(g);
	auto c = mmatching::blossom5::maximum_matching_size(g2.edgeset().begin(), g2.edgeset().end(), g2.N(), g2.edgeset().M());
	BOOST_REQUIRE_EQUAL(c, b);
	for (auto &v : g2) {
		BOOST_CHECK(v.degree > 2);
	}
}

BOOST_AUTO_TEST_CASE(read_weights_test1) {
	std::ifstream f("../data/as20000102.txt.dimacs");
	std::ifstream fw("../dataw/as20000102.txt.dimacs");
	auto g = akt::parse_dimacs(f);
	auto gw = akt::parse_dimacs_with_weights(fw);

	std::ifstream ori("../dataw/as20000102.txt.dimacs");
	std::string str((std::istreambuf_iterator<char>(ori)), std::istreambuf_iterator<char>());
	std::stringstream ss;
	akt::to_dimacs_with_weights(ss, gw);
	BOOST_REQUIRE_EQUAL(str, ss.str());
}
BOOST_AUTO_TEST_CASE(weighted_kernel_test0) {
	std::cout << "weighted_kernel_test0" << std::endl;
	std::string str = R"(p 5 4
e 2 3 9
e 2 4 3
e 3 4 9
e 3 5 3)";
	std::stringstream ss;
	ss.str(str);
	auto g = akt::parse_dimacs_with_weights(ss);
	auto a = mmatching::blossom5::maximum_matching_size(g.edgeset().begin(), g.edgeset().end(), g.N(), g.edgeset().M());
	mmatching::remove_zero_weight_edges(g);
	mmatching::weighted_kernelizer kernel(&g);
	auto s = kernel();
	auto g2 = akt::exclude_isolated_vertices(g);
	for (auto &v : g2) {
		if (v.degree <= 2) {
			akt::to_dimacs_with_weights(std::cout, g2);
			BOOST_REQUIRE_EQUAL(true, false);
		}
	}
	auto b = mmatching::blossom5::maximum_matching_size(g2.edgeset().begin(), g2.edgeset().end(), g2.N(), g2.edgeset().M());
	BOOST_REQUIRE_EQUAL(a, b + s);
}
BOOST_AUTO_TEST_CASE(weighted_kernel_test1) {
	std::cout << "weighted_kernel_test1" << std::endl;
	std::string str = R"(p 3 2
e 1 3 7
e 2 3 4)";
	std::stringstream ss;
	ss.str(str);
	auto g = akt::parse_dimacs_with_weights(ss);
	auto a = mmatching::blossom5::maximum_matching_size(g.edgeset().begin(), g.edgeset().end(), g.N(), g.edgeset().M());
	mmatching::remove_zero_weight_edges(g);
	mmatching::weighted_kernelizer kernel(&g);
	auto s = kernel();
	auto g2 = akt::exclude_isolated_vertices(g);
	for (auto &v : g2) {
		if (v.degree <= 2) {
			BOOST_REQUIRE_EQUAL(true, false);
		}
	}
	auto b = mmatching::blossom5::maximum_matching_size(g2.edgeset().begin(), g2.edgeset().end(), g2.N(), g2.edgeset().M());
	BOOST_REQUIRE_EQUAL(a, b + s);
}
BOOST_AUTO_TEST_CASE(weighted_kernel_test2) {
	std::cout << "weighted_kernel_test2" << std::endl;
	std::string str = R"(p 6 5
e 1 3 4
e 2 3 2
e 2 6 5
e 4 6 1
e 5 6 3)";
	std::stringstream ss;
	ss.str(str);
	auto g = akt::parse_dimacs_with_weights(ss);
	auto a = mmatching::blossom5::maximum_matching_size(g.edgeset().begin(), g.edgeset().end(), g.N(), g.edgeset().M());
	mmatching::remove_zero_weight_edges(g);
	mmatching::weighted_kernelizer kernel(&g);
	auto s = kernel();
	auto g2 = akt::exclude_isolated_vertices(g);
	for (auto &v : g2) {
		if (v.degree <= 2) {
			BOOST_REQUIRE_EQUAL(true, false);
		}
	}
	auto b = mmatching::blossom5::maximum_matching_size(g2.edgeset().begin(), g2.edgeset().end(), g2.N(), g2.edgeset().M());
	BOOST_REQUIRE_EQUAL(a, b + s);
}
BOOST_AUTO_TEST_CASE(weighted_kernel_test3) {
	std::cout << "weighted_kernel_test3" << std::endl;
	std::string str = R"(p 6 4
e 1 6 21
e 2 6 92
e 3 4 7
e 4 5 73)";
	std::stringstream ss;
	ss.str(str);
	auto g = akt::parse_dimacs_with_weights(ss);
	auto a = mmatching::blossom5::maximum_matching_size(g.edgeset().begin(), g.edgeset().end(), g.N(), g.edgeset().M());
	mmatching::remove_zero_weight_edges(g);
	mmatching::weighted_kernelizer kernel(&g);
	auto s = kernel();
	auto g2 = akt::exclude_isolated_vertices(g);
	for (auto &v : g2) {
		if (v.degree <= 2) {
			BOOST_REQUIRE_EQUAL(true, false);
		}
	}
	auto b = mmatching::blossom5::maximum_matching_size(g2.edgeset().begin(), g2.edgeset().end(), g2.N(), g2.edgeset().M());
	BOOST_REQUIRE_EQUAL(a, b + s);
}
BOOST_AUTO_TEST_CASE(weighted_kernel_test4) {
	std::cout << "weighted_kernel_test4" << std::endl;
	std::string str = R"(p 3 3
e 1 2 46
e 1 3 96
e 2 3 45)";
	std::stringstream ss;
	ss.str(str);
	auto g = akt::parse_dimacs_with_weights(ss);
	auto a = mmatching::blossom5::maximum_matching_size(g.edgeset().begin(), g.edgeset().end(), g.N(), g.edgeset().M());
	mmatching::remove_zero_weight_edges(g);
	mmatching::weighted_kernelizer kernel(&g);
	auto s = kernel();
	auto g2 = akt::exclude_isolated_vertices(g);
	for (auto &v : g2) {
		if (v.degree <= 2) {
			BOOST_REQUIRE_EQUAL(true, false);
		}
	}
	auto b = mmatching::blossom5::maximum_matching_size(g2.edgeset().begin(), g2.edgeset().end(), g2.N(), g2.edgeset().M());
	BOOST_REQUIRE_EQUAL(a, b + s);
}
BOOST_AUTO_TEST_CASE(weighted_kernel_test5) {
	std::cout << "weighted_kernel_test5" << std::endl;
	std::string str = R"(p 8 8
e 1 2 39
e 1 7 2
e 3 4 32
e 3 8 65
e 4 8 51
e 5 6 78
e 5 8 35
e 6 8 34)";
	std::stringstream ss;
	ss.str(str);
	auto g = akt::parse_dimacs_with_weights(ss);
	auto a = mmatching::blossom5::maximum_matching_size(g.edgeset().begin(), g.edgeset().end(), g.N(), g.edgeset().M());
	mmatching::remove_zero_weight_edges(g);
	mmatching::weighted_kernelizer kernel(&g);
	auto s = kernel();
	auto g2 = akt::exclude_isolated_vertices(g);
	for (auto &v : g2) {
		if (v.degree <= 2) {
			BOOST_REQUIRE_EQUAL(true, false);
		}
	}
	auto b = mmatching::blossom5::maximum_matching_size(g2.edgeset().begin(), g2.edgeset().end(), g2.N(), g2.edgeset().M());
	BOOST_REQUIRE_EQUAL(a, b + s);
}
BOOST_AUTO_TEST_CASE(weighted_kernel_test6) {
	std::cout << "weighted_kernel_test6" << std::endl;
	std::string str = R"(p 4 5
e 1 2 14
e 1 3 46
e 1 4 12
e 2 4 93
e 3 4 61)";
	std::stringstream ss;
	ss.str(str);
	auto g = akt::parse_dimacs_with_weights(ss);
	auto a = mmatching::blossom5::maximum_matching_size(g.edgeset().begin(), g.edgeset().end(), g.N(), g.edgeset().M());
	mmatching::remove_zero_weight_edges(g);
	mmatching::weighted_kernelizer kernel(&g);
	auto s = kernel();
	auto g2 = akt::exclude_isolated_vertices(g);
	for (auto &v : g2) {
		if (v.degree <= 1) {
			BOOST_REQUIRE_EQUAL(true, false);
		} else if(v.degree == 2) {
			auto x = v.neighbors.begin()->id;
			auto y = (++v.neighbors.begin())->id;
			BOOST_CHECK(std::find_if(g2[x].neighbors.begin(), g2[x].neighbors.end(), [&] (auto& w) { return w.id == y; }) != g2[x].neighbors.end());
		}
	}
	auto b = mmatching::blossom5::maximum_matching_size(g2.edgeset().begin(), g2.edgeset().end(), g2.N(), g2.edgeset().M());
	BOOST_REQUIRE_EQUAL(a, b + s);
}

BOOST_AUTO_TEST_CASE(weighted_kernel_test7) {
	std::cout << "weighted_kernel_test7" << std::endl;
	std::string str = R"(p 11 14
e 1 5 26
e 1 6 95
e 1 7 66
e 2 10 87
e 3 4 44
e 3 9 71
e 3 11 73
e 5 7 95
e 5 10 19
e 6 9 20
e 7 10 12
e 8 10 39
e 9 11 60
e 10 11 9)";
	std::stringstream ss;
	ss.str(str);
	auto g = akt::parse_dimacs_with_weights(ss);
	auto a = mmatching::blossom5::maximum_matching_size(g.edgeset().begin(), g.edgeset().end(), g.N(), g.edgeset().M());
	mmatching::remove_zero_weight_edges(g);
	mmatching::weighted_kernelizer kernel(&g);
	auto s = kernel();
	auto g2 = akt::exclude_isolated_vertices(g);
	for (auto &v : g2) {
		if (v.degree <= 1) {
			BOOST_REQUIRE_EQUAL(true, false);
		} else if(v.degree == 2) {
			auto x = v.neighbors.begin()->id;
			auto y = (++v.neighbors.begin())->id;
			BOOST_CHECK(std::find_if(g2[x].neighbors.begin(), g2[x].neighbors.end(), [&] (auto& w) { return w.id == y; }) != g2[x].neighbors.end());
		}
	}
	auto b = mmatching::blossom5::maximum_matching_size(g2.edgeset().begin(), g2.edgeset().end(), g2.N(), g2.edgeset().M());
	BOOST_REQUIRE_EQUAL(a, b + s);
}

BOOST_AUTO_TEST_CASE(weighted_kernel_test8) {
	std::cout << "weighted_kernel_test8" << std::endl;
	std::string str = R"(p 14 15
e 1 3 76
e 1 8 35
e 2 10 43
e 3 8 63
e 4 13 59
e 5 6 87
e 5 10 58
e 5 12 92
e 5 13 71
e 6 14 91
e 7 8 85
e 7 14 55
e 9 10 7
e 10 11 94
e 11 12 22)";
	std::stringstream ss;
	ss.str(str);
	auto g = akt::parse_dimacs_with_weights(ss);
	auto a = mmatching::blossom5::maximum_matching_size(g.edgeset().begin(), g.edgeset().end(), g.N(), g.edgeset().M());
	mmatching::remove_zero_weight_edges(g);
	mmatching::weighted_kernelizer kernel(&g);
	auto s = kernel();
	auto g2 = akt::exclude_isolated_vertices(g);
	for (auto &v : g2) {
		if (v.degree <= 1) {
			BOOST_REQUIRE_EQUAL(true, false);
		} else if(v.degree == 2) {
			auto x = v.neighbors.begin()->id;
			auto y = (++v.neighbors.begin())->id;
			BOOST_CHECK(std::find_if(g2[x].neighbors.begin(), g2[x].neighbors.end(), [&] (auto& w) { return w.id == y; }) != g2[x].neighbors.end());
		}
	}
	auto b = mmatching::blossom5::maximum_matching_size(g2.edgeset().begin(), g2.edgeset().end(), g2.N(), g2.edgeset().M());
	BOOST_REQUIRE_EQUAL(a, b + s);
}


BOOST_AUTO_TEST_CASE(rnd_kernel_test1) {
	std::cout << "rnd_kernel_test1" << std::endl;
	std::mt19937 rng;
	rng.seed(std::random_device()());
	std::uniform_int_distribution<std::mt19937::result_type> dist(1, 50); //
	size_t as = 0;
	size_t ssum = 0;
	for (int i = 0; i < 100; i++) {
		for (int j = 0; j < 500; j++) {
		
			auto g = akt::generate_weighted_graph(i, dist(rng),100);
			std::stringstream ss;
			akt::to_dimacs_with_weights(ss, g);
			auto a = mmatching::blossom5::maximum_matching_size(g.edgeset().begin(), g.edgeset().end(), g.N(), g.edgeset().M());
			as += a;
			mmatching::remove_zero_weight_edges(g);
			mmatching::weighted_kernelizer kernel(&g);
			auto s = kernel();
			ssum += s;
			auto g2 = akt::exclude_isolated_vertices(g);
			for (auto &v : g2) {
				if (v.degree <= 1) {
					BOOST_REQUIRE_EQUAL(true, false);
				} else if(v.degree == 2) {
					auto x = v.neighbors.begin()->id;
					auto y = (++v.neighbors.begin())->id;
					BOOST_CHECK(std::find_if(g2[x].neighbors.begin(), g2[x].neighbors.end(), [&] (auto& w) { return w.id == y; }) != g2[x].neighbors.end());
				}
			}
			auto b = mmatching::blossom5::maximum_matching_size(g2.edgeset().begin(), g2.edgeset().end(), g2.N(), g2.edgeset().M());
			if (a != b+s) {
				std::cout << ss.str() << std::endl;
			}
			BOOST_REQUIRE_EQUAL(a, b + s);
		}
	}
	std::cout << as << std::endl;
	std::cout << ssum << std::endl;
}
/*
BOOST_AUTO_TEST_CASE(weighted_kernel_test) {
	std::ifstream f("../dataw/as20000102.txt.dimacs");
	auto g = akt::parse_dimacs_with_weights(f);
	auto a = mmatching::blossom5::maximum_matching_size(g.edgeset().begin(), g.edgeset().end(), g.N(), g.edgeset().M());
	mmatching::remove_zero_weight_edges(g);
	mmatching::weighted_kernelizer kernel(&g);
	auto s = kernel();
	auto g2 = akt::exclude_isolated_vertices(g);
	for (auto &v : g2) {
		if (v.degree <= 1) {
			BOOST_REQUIRE_EQUAL(true, false);
		}
	}
	auto b = mmatching::blossom5::maximum_matching_size(g2.edgeset().begin(), g2.edgeset().end(), g2.N(), g2.edgeset().M());
	BOOST_REQUIRE_EQUAL(a, b + s);
} //*/
